-- The frame code and fixes are by Vrul (really much appreciated) for gmSpecs
-- You can check the forum here:
-- http://www.wowinterface.com/forums/showthread.php?t=54975

local ADDON = ...

local QUESTION_MARK_ICON = [[Interface\Icons\INV_MISC_QUESTIONMARK]]
local NOTASET = "No Set"

local fontName, fontHeight, fontFlags = GameFontNormal:GetFont()

local BUTTON_HEIGHT = fontHeight + 4
local BUTTON_SPACING = 0
local MENU_BUFFER = 10
local MENU_SPACING = 1
local MENU_WIDTH_EMPTY = 190
local MENU_WIDTH = 200

local GMES_CFG = {
		maxsets = 10,    -- max num of sets defined by game
}

-- upvalues to be used perhaps in the future.
local string_format = string.format
local string_find = string.find
local string_sub = string.sub

local Menu = CreateFrame('Frame', nil, UIParent)
Menu:SetFrameStrata('TOOLTIP')
Menu:SetClampedToScreen(true)
Menu:SetBackdrop({
	bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
	tile = true, tileSize = 32, edgeSize = 16,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
})
Menu:Hide()


local function initMenu()		
		for index = 1, GMES_CFG["maxsets"] do
			Menu[index]:SetText("")
			Menu[index]:Hide()
		end		
		Menu:SetSize(MENU_WIDTH, (MENU_BUFFER * 5) + (fontHeight * 3) + MENU_SPACING + ((BUTTON_HEIGHT + BUTTON_SPACING) - BUTTON_SPACING))
end

local function Menu_OnLeave()
	local focus = GetMouseFocus() or WorldFrame
	if focus ~= Menu and focus:GetParent() ~= Menu then
		Menu:Hide()
	end
end
Menu:SetScript('OnLeave', Menu_OnLeave)

-- Header
local header_desc = Menu:CreateFontString()
header_desc:SetPoint("TOPLEFT", Menu, "TOPLEFT", MENU_BUFFER, -MENU_BUFFER)
header_desc:SetFont(fontName, fontHeight)
header_desc:SetTextColor(1, 1, 1, 1)
header_desc:SetText("Equipment Sets:")

local header_value = Menu:CreateFontString()
header_value:SetPoint("TOPRIGHT", Menu, "TOPRIGHT", -MENU_BUFFER, -MENU_BUFFER)
header_value:SetFont(fontName, fontHeight)
header_value:SetTextColor(1, 1, 1, 1)
header_value:SetText("")

-- Right click
local rclick_desc = Menu:CreateFontString()
rclick_desc:SetPoint("BOTTOMLEFT", Menu, "BOTTOMLEFT", MENU_BUFFER, MENU_BUFFER)
rclick_desc:SetFont(fontName, fontHeight)
rclick_desc:SetTextColor(0, 1, 0, 1)
rclick_desc:SetText("Right-Click")

local rclick_value = Menu:CreateFontString()
rclick_value:SetPoint("BOTTOMRIGHT", Menu, "BOTTOMRIGHT", -MENU_BUFFER, MENU_BUFFER)
rclick_value:SetFont(fontName, fontHeight)
rclick_value:SetTextColor(1, 0.8, 0, 1)
rclick_value:SetText("Set Manager")

-- Left click
local lclick_desc = Menu:CreateFontString()
lclick_desc:SetPoint("BOTTOMLEFT", rclick_desc, "TOPLEFT", 0, MENU_SPACING)
lclick_desc:SetFont(fontName, fontHeight)
lclick_desc:SetTextColor(0, 1, 0, 1)
lclick_desc:SetText("Left-Click")

local lclick_value = Menu:CreateFontString()
lclick_value:SetPoint("BOTTOMRIGHT", rclick_value, "TOPRIGHT", 0, MENU_SPACING)
lclick_value:SetFont(fontName, fontHeight)
lclick_value:SetTextColor(1, 0.8, 0, 1)
lclick_value:SetText("Equip Set")


local dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(ADDON, {
	type = "data source",
	icon = "Interface\\Addons\\"..ADDON.."\\icon.png",
	text = ADDON,
	OnClick = function(self, button)
		if button == "RightButton" then
			ToggleCharacter('PaperDollFrame')
			PaperDollFrame_SetSidebar(nil, 3)
		end
		Menu:SetShown(not Menu:IsShown())					
	end,
	OnEnter = function(self)
		if not Menu:IsShown() then
			local _, selfCenter = self:GetCenter()
			local _, uiCenter = UIParent:GetCenter()
			Menu:ClearAllPoints()
			if selfCenter >= uiCenter then
				Menu:SetPoint('TOP', self, 'BOTTOM')
			else
				Menu:SetPoint('BOTTOM', self, 'TOP')
			end
			Menu:Show()
		end
	end,
	OnLeave = Menu_OnLeave
})

local function UpdateLDB()
	for index=1, C_EquipmentSet.GetNumEquipmentSets() do
		local name, icon, setID, isEquipped, numItems, numEquipped, numInventory, numMissing, numIgnored = GetEquipmentSetInfo(index)		
		if isEquipped == true then 		
			dataobj.text = name
			dataobj.icon = icon
			return
		end
	end	
	dataobj.text = NOTASET
	dataobj.icon = QUESTION_MARK_ICON
end


local function Button_OnClick(self, button, down)
	if button == "RightButton" then
		ToggleCharacter('PaperDollFrame')
		PaperDollFrame_SetSidebar(nil, 3)
	elseif button == "LeftButton" then
		UseEquipmentSet(GetEquipmentSetInfo(self.index));
		return
	elseif button == "MiddleButton" then	
		return
	end
	Menu:Hide()
end

Menu:RegisterEvent("PLAYER_LOGIN")
Menu:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
Menu:RegisterEvent('UNIT_INVENTORY_CHANGED')
Menu:RegisterEvent('EQUIPMENT_SETS_CHANGED')

Menu:SetScript("OnEvent", function(self, event, ...)

	if event == "PLAYER_LOGIN" then	
		Menu:UnregisterEvent(event)				
		for index = 1, GMES_CFG["maxsets"] do	
		
			local button = CreateFrame("Button", nil, Menu)
			if index ~= 1 then
				button:SetPoint("TOPLEFT", Menu[index - 1], "BOTTOMLEFT", 0, -BUTTON_SPACING)
			else
				button:SetPoint("TOPLEFT", header_desc, "BOTTOMLEFT", 0, -MENU_BUFFER)
			end
			
			button:SetPoint("RIGHT", -MENU_BUFFER, 0)
			button:SetHeight(BUTTON_HEIGHT)
			button:SetNormalFontObject("GameFontNormal")
			button:SetHighlightTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
			button.index = index
			button:RegisterForClicks("AnyDown")
			button:SetScript("OnClick", Button_OnClick)
			button:SetScript("OnLeave", Menu_OnLeave)

			local text = button:CreateFontString(ADDON .. "btn_font", nil, "GameFontNormal")
			text:SetAllPoints()
			text:SetJustifyH("LEFT")
			text:SetJustifyV("MIDDLE")
			text:SetTextColor(1, 1, 1, 1)
			
			button:SetFontString(text)
			Menu[index] = button
			
		end		
	end 
	
	initMenu()
	
	local numButtons = C_EquipmentSet.GetNumEquipmentSets()
	for index = 1, numButtons do
		local name, icon, setID, isEquipped, numItems, numEquipped, numInventory, numMissing, numIgnored = GetEquipmentSetInfo(index)
		local items = numItems .. ":" .. numEquipped
		Menu[index]:SetText("")
		Menu[index]:SetText(("|T%s:0|t %-6s %s"):format(icon, items, name))
		Menu[index]:Show()
	end 
	
	Menu:SetSize(MENU_WIDTH, (MENU_BUFFER * 5) + (fontHeight * 3) + MENU_SPACING + ((BUTTON_HEIGHT + BUTTON_SPACING) * numButtons - BUTTON_SPACING))	
	UpdateLDB()

end)